#include "testlib.h"
#include <iostream>
#include <vector>
#include <cassert>

using namespace std;

int maxC = 1'000'000'000;
int n;

int my_rand(int l, int r) {
    assert(l <= r);
    if (l == r)
        return l;
    return rnd.wnext(r - l + 1, 0) + l;
}

void print(vector<pair<int, int>> output) {
    cout << n << "\n";
    for (int i = 0; i < n; i++)
        cout << output[i].first << " \n"[i == n - 1];
    for (int i = 0; i < n; i++)
        cout << output[i].second << " \n"[i == n - 1];
}

void standard() {
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n; i++) {
        int a = my_rand(1, maxC), b = my_rand(1, maxC);
        if (a < b)
            swap(a, b);
        output[i] = {a, b};
    }
    print(output);
}

void group4() {
    vector<int> numbers(2 * n);
    set<int> was;
    for (int &i: numbers) {
        while (true) {
            i = my_rand(1, maxC);
            if (!was.count(i)) {
                break;
            }
        }
        was.insert(i);
    }
    sort(numbers.begin(), numbers.end());
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n; i++) {
        output[i] = {numbers[2 * i + 1], numbers[2 * i]};
    }
    print(output);
}

void partition() {
    vector<pair<int, int>> output(n);
    int pivot = my_rand(1, maxC);
    for (int i = 0; i < n; i++)
        output[i] = {my_rand(pivot, maxC), my_rand(1, pivot)};
    print(output);
}

void best_answer_at_start() {
    vector<pair<int, int>> output(n);
    output[0] = {maxC, 1};
    for (int i = 1; i < n; i++) {
        output[i] = {my_rand(1, maxC), my_rand(1, maxC)};
        if (output[i].first < output[i].second) {
            swap(output[i].first, output[i].second);
        }
    }
    print(output);
}

bool flip_coin(int success, int all) {
    // return true with probability success/all
    return my_rand(1, all) <= success;
}

void group5() {
    vector<int> numbers(2 * n);
    for (int &i: numbers) {
        i = my_rand(1, maxC);
    }
    sort(numbers.begin(), numbers.end());
    vector<pair<int, int>> output(n);
    int pnt_a = 0, pnt_b = 0;
    for (int i = 0; i < 2 * n; i++) {
        if (pnt_a == pnt_b) {
            assert(pnt_b < n);
            output[pnt_b++].second = numbers[i];
            continue;
        }
        if (flip_coin(n - pnt_b, n - pnt_a + n - pnt_b)) {
            output[pnt_b++].second = numbers[i];
        } else {
            output[pnt_a++].first = numbers[i];
        }
    }
    print(output);
}

void merge_at_the_end() {
    vector<int> numbers(2 * n - 2);
    for (int &i: numbers) {
        i = my_rand(1, maxC);
    }
    sort(numbers.begin(), numbers.end());
    vector<pair<int, int>> output(n);
    int mn = maxC, mx = 0;
    for (int i = 0; i < n - 1; i++) {
        output[i] = {numbers[i * 2 + 1], numbers[i * 2]};
        mn = min(mn, numbers[i * 2]);
        mx = max(mx, numbers[i * 2 + 1]);
    }
    for (int i = 0; i < n - 1; i++) {
        if (numbers[i * 2] == mn) {
            output[n - 1].second = numbers[i * 2 + 1];
        }
        if (numbers[i * 2 + 1] == mx) {
            output[n - 1].first = numbers[i * 2];
        }
    }
    print(output);
}

void connectivity() {
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n / 2; i++) {
        output[i] = {i * 2 + 2, i * 2 + 1};
    }
    for (int i = n / 2; i < n; i++) {
        output[i] = {(i - n / 2) * 2 + 3, (i - n / 2) * 2 + 2};
    }
    for (int i = n / 2; i < n; i++) {
        swap(output[i], output[my_rand(n / 2, i)]);
    }
    print(output);
}

void random_connectivity() {
    vector<pair<int, int>> candidates;
    for (int i = 1; i <= n * 3; i++) {
        candidates.push_back({i + 1, i});
    }
    for (int i = 0; i < candidates.size(); i++) {
        swap(candidates[i], candidates[my_rand(0, i)]);
    }
    while (candidates.size() > n)
        candidates.pop_back();
    print(candidates);
}

void random_connectivity_group4() {
    vector<pair<int, int>> output;
    for (int i = 0; i < n / 2; i++) {
        output.push_back({i * 2 + 2, i * 2 + 1});
    }
    vector<pair<int, int>> candidates;
    for (int i = 0; i < n; i++) {
        candidates.push_back({i * 2 + 3, i * 2 + 2});
        swap(candidates[i], candidates[my_rand(0, i)]);
    }
    while (output.size() < n) {
        output.push_back(candidates.back());
        candidates.pop_back();
    }
    sort(output.begin(), output.end());
    print(output);
}

void group5_connectivity() {
    vector<int> numbers;
    for (int i = 0; i < 2 * n; i++) {
        numbers.push_back(my_rand(1, maxC));
    }
    vector<pair<int, int>> output;
    sort(numbers.begin(), numbers.end());
    for (int i = 0; i < n; ) {
        int pnt = i;
        while(pnt + 1 < n && my_rand(1, 8) != 1) { // 8 is just a random small number
            ++pnt;
        }
        for (int j = i; j <= pnt; j++) {
            output.push_back({numbers[i * 2 + pnt - i + 1 + j - i], numbers[i * 2 + j - i]});
        }
        i = pnt + 1;
    }
    print(output);
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    int maxN = opt<int>("maxN");
    n = opt<int>("n");
    maxC = opt<int>("maxC", maxC);
    auto mode = opt<string>("mode", "standard");

    int t = maxN / n;

    cout << t << '\n';

    for (int _ = 0; _ < t; _++) {
        if (mode == "standard") {
            standard();
        } else if (mode == "group4") {
            group4();
        } else if (mode == "group5") {
            group5();
        } else if (mode == "partition") {
            partition();
        } else if (mode == "merge_at_the_end") {
            merge_at_the_end();
        } else if (mode == "best_answer_at_start") {
            best_answer_at_start();
        } else if (mode == "connectivity") {
            connectivity();
        } else if (mode == "random_connectivity") {
            random_connectivity();
        } else if (mode == "random_connectivity_group4") {
            random_connectivity_group4();
        } else if (mode == "group5_connectivity") {
            group5_connectivity();
        }
    }

    return 0;
}
