#include "testlib.h"
#include <iostream>
#include <vector>
#include <cassert>

using namespace std;

int maxC = 1'000'000'000;
int maxN;
int n;
bool do_shuffle = false;

int my_rand(int l, int r) {
    assert(l <= r);
    if (l == r)
        return l;
    return rnd.wnext(r - l + 1, 0) + l;
}

void print(vector<pair<int, int>> output, bool confirm_shuffle = true) {
    if (do_shuffle && confirm_shuffle) {
        for (int i = 1; i < n; i++) {
            swap(output[i], output[my_rand(0, i - 1)]);
        }
    }

    cout << n << "\n";
    for (int i = 0; i < n; i++)
        cout << output[i].first << " \n"[i == n - 1];
    for (int i = 0; i < n; i++)
        cout << output[i].second << " \n"[i == n - 1];
}

void embedded() {
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n; i++) {
        output[i] = {2 * n - i, i + 1};
    }
    reverse(output.begin(), output.end());
    print(output);
}

void step1() {
    // group 5
    int size = opt<int>("size", n);
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n; i++) {
        output[i] = {size + i + 1, i + 1};
    }
    print(output);
}

void without_intersection() {
    // group 4, 5
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n; i++) {
        output[i] = {i * 2 + 2, i * 2 + 1};
    }
    print(output);
}

void binary_tree() {
    // it is reasonable to do this test only with shuffle
    vector<pair<int, int>> output;
    for (int i = 1; i < n; i *= 2) {
        for (int j = 0; j < n; j += i) {
            output.emplace_back(i + j + 1, j + 1);
        }
    }
    print(output);
}

void growing_segments() {
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n / 2; i++) {
        output[i] = {i * 2 + 2, i * 2 + 1};
        if (do_shuffle) {
            swap(output[i], output[my_rand(0, i)]);
        }
    }
    vector<int> ord;
    for (int i = 0; i < n / 2; i++) {
        if (i % 2 == 0) {
            ord.push_back(n / 4 + i / 2);
        } else {
            ord.push_back(n / 4 - 1 - i / 2);
        }
    }
    int l = 1e9, r = 1;
    for (int i = 0; i < n / 2; i++) {
        l = min(l, output[ord[i]].second);
        r = max(r, output[ord[i]].first);
        output[n / 2 + i] = {r, l};
    }
    print(output, false);
}

template<typename T>
vector<T> pyramid_order(const vector<T>& a) {
    vector<T> result;
    for (int i = 0; i < a.size(); i++) {
        if (i % 2 == 0) {
            result.push_back(a[a.size() / 2 + i / 2]);
        } else {
            result.push_back(a[a.size() / 2 - 1 - i / 2]);
        }
    }
    return result;
}

void special_connectivity() {
    vector<pair<int, int>> output(n);
    for (int i = 0; i < n / 2; i++) {
        output[i] = {i * 2 + 2, i * 2 + 1};
    }
    vector<pair<int, int>> output2;
    for (int i = n / 2; i < n; i++) {
        output2.push_back({(i - n / 2) * 2 + 3, (i - n / 2) * 2 + 2});
    }
    output2 = pyramid_order(output2);
    for (int i = n / 2; i < n; i++) {
        output[i] = output2[i - n / 2];
    }
    print(output);
}

void connectivity_segment_group4() {
    int k = n / 10;
    vector<pair<int, int>> output;
    for (int i = 0; i < n - k; i++) {
        output.push_back({i * 2 + 2, i * 2 + 1});
    }
    int pos = my_rand(k, n - k - k - 1);
    for (int i = pos; i < pos + k; i++) {
        output.push_back({i * 2 + 3, i * 2 + 2});
    }
    sort(output.begin(), output.end());
    print(output);
}

void ormlis_hack() {
    vector<pair<int, int>> output;
    for (int i = 0; i < n - 1; i++) {
        output.push_back({i * 2 + 2, i * 2 + 1});
    }
    output.push_back({(n / 2) * 2 + 3, (n / 2) * 2 + 2});
    print(output);
}

void all() {
    int t = maxN / n;
    // cout << t << endl;
    int cur = t;
    vector<int> numbers;
    for (int i = 0; i < n * 2; i++) {
        numbers.push_back(my_rand(1, maxC));
    }
    string extra_mode = opt<string>("extra_mode", "none");
    sort(numbers.begin(), numbers.end());
    vector<vector<pair<int, int>>> output;
    for (int i = 0; i < (1 << (2 * n)) && cur; i++) {
        if (__builtin_popcount(i) != n)
            continue;
        vector<int> a, b;
        for (int j = 0; j < 2 * n; j++) {
            if ((1 << j) & i) {
                b.push_back(numbers[j]);
            } else {
                a.push_back(numbers[j]);
            }
        }
        bool bad = false;
        for (int j = 0; j < n; j++) {
            if (b[j] > a[j]) {
                bad = true;
            }
        }
        if (extra_mode == "group4") {
            for (int j = 0; j + 1 < n; j++) {
                if (a[j] >= b[j + 1]) {
                    bad = true;
                }
            }
        } else if (extra_mode == "group5") {
            for (int j = 0; j + 1 < n; j++) {
                if (a[j] > a[j + 1] || b[j] > b[j + 1]) {
                    bad = true;
                }
            }
        }
        if (bad)
            continue;
        vector<pair<int, int>> current;
        for (int j = 0; j < n; j++) {
            current.push_back({a[j], b[j]});
        }
        output.push_back(current);
        cur -= 1;
    }
    cout << output.size() << endl;
    for (int i = 0; i < output.size(); i++) {
        print(output[i]);
    }
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    maxN = opt<int>("maxN");
    n = opt<int>("n");
    maxC = opt<int>("maxC", maxC);
    auto mode = opt<string>("mode", "standard");
    do_shuffle = opt<bool>("shuffle", false);

    if (mode == "all") {
        all();
        return 0;
    }
    
    int t = maxN / n;

    cout << t << '\n';

    for (int _ = 0; _ < t; _++) {
        if (mode == "embedded") {
            embedded();
        } else if (mode == "step1") {
            step1();
        } else if (mode == "without_intersection") {
            without_intersection();
        } else if (mode == "binary_tree") {
            binary_tree();
        } else if (mode == "growing_segments") {
            growing_segments();
        } else if (mode == "special_connectivity") {
            special_connectivity();
        } else if (mode == "connectivity_segment_group4") {
            connectivity_segment_group4();
        } else if (mode == "ormlis_hack") {
            ormlis_hack();
        }
    }

    return 0;
}
